char *help = "\n"
"\n"
" EBC2ASCB - Translate an EBCIDC stream to ASCII.\n"
"            (attempt to replace control codes with control codes)\n"
"\n"
" usage:\n"
"       EBC2ASCB [{<text_file>}]\n";

#include <stdio.h>
#include <stdlib.h>

unsigned char table[256] = {

   '\x00','\x01','\x02','\x03','\x04','\x05','\x06','\x07',
/*   NUL    SOH    STX    ETX    SEL    HT     RNL    DEL  */
   '\x08','\x09','\x0A','\x0B','\x0C','\x0D','\x0E','\x0F',
/*   GE     SPS    RPT     VT    FF     CR     SO     SI   */
   '\x10','\x11','\x12','\x13','\x14','\x15','\x16','\x17',
/*   DLE    DC1    DC2    DC3  RES/ENP  NL     BS     POC  */
   '\x18','\x19','\x1A','\x1B','\x1C','\x1D','\x1E','\x1F',
/*   CAN    EM     UBS    CU1    IFS    IGS    IRS  ITB/IUS*/
   '\x20','\x21','\x22','\x23','\x24','\x25','\x26','\x27',
/*   DS     SOS    FS     WUS  BYP/INP  LF     ETB    ESC  */
   '\x28','\x29','\x2A','\x2B','\x2C','\x2D','\x2E','\x2F',
/*   SA     SFE   SM/SW   CSP    MFA    ENQ    ACK    BEL  */
   '\x30','\x31','\x32','\x33','\x34','\x35','\x36','\x37',
/*                 SYN    IR     PP     TRN    NBS    EOT  */
   '\x38','\x39','\x3A','\x3B','\x3C','\x3D','\x3E','\x3F',
/*   SBS    IT     RFF    CU3    DC4    NAK           SUB  */
   '\x20','\x41','\x42','\x43','\x44','\x45','\x46','\x47',
/*   SPC                                                   */
   '\x48','\x49','\x4A','\x2E','\x3C','\x4D','\x4E','\x4F',
/*                       .      <      (      +      |    */
   '\x50','\x51','\x52','\x53','\x54','\x55','\x56','\x57',
/*   &                                                     */
   '\x58','\x59','\x21','\x24','\x5C','\x5D','\x3B','\x5F',
/*                 !      $      *      )      ;      ^    */
   '\x2D','\x2F','\x62','\x63','\x64','\x65','\x66','\x67',
/*   -      /                                              */
   '\x68','\x69','\x6A','\x2C','\x6C','\x6D','\x3E','\x6F',
/*                 |      ,      %      _      >      ?    */
   '\x70','\x71','\x72','\x73','\x74','\x75','\x76','\x77',
/*                                                         */
   '\x78','\x79','\x3A','\x23','\x7C','\x7D','\x3D','\x7F',
/*          `      :      #      @      '      =      "    */
   '\x80','\x61','\x62','\x63','\x64','\x65','\x66','\x67',
/*          a      b      c      d      e      f      g    */
   '\x68','\x69','\x8A','\x8B','\x8C','\x8D','\x8E','\x8F',
/*   h      i                                              */
   '\x90','\x6A','\x6B','\x6C','\x6D','\x6E','\x6F','\x70',
/*          j       k     l      m      n      o      p    */
   '\x71','\x72','\x9A','\x9B','\x9C','\x9D','\x9E','\x9F',
/*   q      r                                              */
   '\xA0','\x7E','\x73','\x74','\x75','\x76','\x77','\x78',
/*          ~      s      t      u      v      w      x    */
   '\x79','\x7A','\xAA','\xAB','\xAC','\x5B','\xAE','\xAF',
/*   y      z                           [                   */
   '\xB0','\xB1','\xB2','\xB3','\xB4','\xB5','\xB6','\xB7',
/*                                                         */
   '\xB8','\xB9','\xBA','\xBB','\xBC','\x5D','\xBE','\xBF',
/*                                      ]                   */
   '\x7B','\x41','\x42','\x43','\x44','\x45','\x46','\x47',
/*   {      A      B      C      D      E      F      G    */
   '\x48','\x49','\xC2','\xC3','\xC4','\xC5','\xC6','\xC7',
/*   H      I                                              */
   '\x7D','\x4A','\x4B','\x4C','\x4D','\x4E','\x4F','\x50',
/*   }      J      K      L      M      N      O      P    */
   '\x51','\x52','\xD2','\xD3','\xD4','\xD5','\xD6','\xD7',
/*   Q      R                                              */
   '\x5C','\xD9','\x53','\x54','\x55','\x56','\x57','\x58',
/*   \             S      T      U       V     W      X    */
   '\x59','\x5A','\xE2','\xE3','\xE4','\xE5','\xE6','\xE7',
/*   Y      Z                                              */
   '\x30','\x31','\x32','\x33','\x34','\x35','\x36','\x37',
/*   0      1      2      3      4      5      6      7    */
   '\x38','\x39','\xFA','\xFB','\xFC','\xFD','\xFE','\xFF'
/*   8      9      |                                       */
};
                                                           
void  ebc2asc(void);
int   main (int, char *[]);


void  ebc2asc()
{                                                          
   unsigned char  c;
                                                           
   while ( (c = (unsigned char) getchar() ) != 0x00ff )
      putchar( table[c] );                                 
}
                                                           

                                                           

int main (                                                 
int   argc,
char *argv[]                                               
)
{                                                          
   /* parse command line options */
   while ( --argc > 0  &&  (*++argv)[0] == '-' )           
      switch ( (*argv)[1] )  {
      case '?':
      default:
         fprintf( stderr, "%s", help );
         exit( 1 );
      }

   if ( argc > 0 )
      while ( argc-- )  {
         if ( freopen( *argv, "rb", stdin ) == NULL )  {
            fprintf( stderr,"EBC2ASC: cannot open '%s'\n", *argv );
            exit( 1 );
         }
         ebc2asc();
         ++argv;
      }
   else
      ebc2asc();

   return( 0 );
}

